import logging
import ssl
from typing import Awaitable

import certifi
import websockets
from websockets.exceptions import ConnectionClosedError

from hasherino.parse_irc import ParsedMessage


class TwitchWebsocket:
    def __init__(self) -> None:
        self._websocket = None

    async def is_connected(self) -> bool:
        return self._websocket is not None

    async def _authenticate(self, token: str, user: str):
        """
        Returns parsed authentication response
        """
        if self._websocket is None:
            raise Exception("Websocket not connected")

        await self._websocket.send(f"PASS oauth:{token}")
        await self._websocket.send(f"NICK {user}")
        await self._websocket.send(f"USER {user} 8 * :{user}")

    async def join_channel(self, channel: str):
        if self._websocket is None:
            raise Exception("Websocket not connected")

        await self._websocket.send(f"JOIN #{channel}")

    async def leave_channel(self, channel: str):
        if self._websocket is None:
            raise Exception("Websocket not connected")

        await self._websocket.send(f"PART #{channel}")

    async def send_message(self, channel: str, message: str):
        logging.debug(f"Sending message on channel {channel} message: {message}")

        if self._websocket is None:
            raise Exception("Websocket not connected")

        logging.debug("Acquired lock, sending message")
        await self._websocket.send(f"PRIVMSG #{channel} :{message}")

    async def listen_message(
        self,
        message_callback: Awaitable,
        reconnect_callback: Awaitable[bool],
        token: str,
        username: str,
        join_channel: str | None = None,
    ):
        ssl_context = ssl.create_default_context(cafile=certifi.where())
        async for websocket in websockets.connect(
            "wss://irc-ws.chat.twitch.tv:443",
            ping_interval=3,
            ping_timeout=2,
            ssl=ssl_context,
        ):
            try:
                self._websocket = websocket

                await websocket.send(
                    "CAP REQ :twitch.tv/commands twitch.tv/membership twitch.tv/tags"
                )
                await self._authenticate(token, username)
                await reconnect_callback(False)

                if join_channel:
                    await self.join_channel(join_channel)

                async for message in websocket:
                    try:
                        parsed_message: ParsedMessage = ParsedMessage(message)
                        await message_callback(parsed_message)
                    except Exception as e:
                        logging.exception(e)

            except ConnectionClosedError:
                logging.warning("Websocket connection closed, reconnecting")
                self._websocket = None
                await reconnect_callback(True)

            except Exception as e:
                logging.exception(f"Websocket connection failed: {e}")



from abc import ABC, abstractmethod

import flet as ft
import validators

from hasherino.hasherino_dataclasses import Emote, Message
from hasherino.pubsub import PubSub


class FontSizeSubscriber(ABC):
    @abstractmethod
    async def on_font_size_changed(self, new_font_size: int):
        ...


class ShowTimestampSubscriber(ABC):
    @abstractmethod
    async def on_show_timestamp_changed(self, show_timestamp: bool):
        ...


class ChatText(ft.Container, FontSizeSubscriber):
    def __init__(self, text: str, color: str, size: int, weight=""):
        try:
            is_url = validators.url(text)
        except validators.ValidationError:
            is_url = False

        color = ft.colors.BLUE if is_url else color
        self.content = ft.Text(
            text, size=size, weight=weight, color=color, selectable=True
        )
        super().__init__(content=self.content, url=text)

    async def on_font_size_changed(self, new_font_size: int):
        self.content.size = new_font_size


class ChatBadge(ft.Image, FontSizeSubscriber):
    def __init__(self, src: str, height: int):
        super().__init__(src=src, height=height)

    async def on_font_size_changed(self, new_font_size: int):
        self.height = new_font_size


class ChatEmote(ft.Container, FontSizeSubscriber):
    def __init__(self, *args, **kwargs):
        self.emote: Emote | None = kwargs.pop("emote", None)
        super().__init__(
            content=ft.Image(
                tooltip=self.emote.name if self.emote else "", *args, **kwargs
            ),
        )

    async def on_font_size_changed(self, new_font_size: int):
        self.height = new_font_size * 2


class ChatTimestamp(ft.Text, ShowTimestampSubscriber, FontSizeSubscriber):
    def __init__(self, text: str, color: str, size: int):
        super().__init__(text, size=size, color=color, selectable=False)

    async def on_show_timestamp_changed(self, show_timestamp: bool):
        self.visible = bool(show_timestamp)

    async def on_font_size_changed(self, new_font_size: int):
        self.size = max(new_font_size - 4, 4)


class ChatMessage(ft.Row):
    def __init__(self, message: Message, page: ft.Page, font_size: int):
        super().__init__()
        self.vertical_alignment = "start"
        self.wrap = True
        self.width = page.width
        self.page = page
        self.font_size = font_size
        self.spacing = 2
        self.run_spacing = 0
        self.vertical_alignment = ft.CrossAxisAlignment.CENTER

        self.add_control_elements(message)

    def add_control_elements(self, message):
        if message.timestamp is not None:
            self.controls.append(
                ChatTimestamp(
                    text=f"{message.timestamp.strftime('%H:%M')} ",
                    color=ft.colors.GREY,
                    size=max(self.font_size - 4, 4),
                )
            )

        self.controls.extend(
            [ChatBadge(badge.url, self.font_size) for badge in message.user.badges]
        )

        self.controls.append(
            ChatText(
                f"{message.user.name}: ",
                message.user.chat_color,
                self.font_size,
                weight="bold",
            )
        )

        for element in message.elements:
            if type(element) is str:
                color = message.user.chat_color if message.me else ""
                result = ChatText(element, color, self.font_size)
            elif type(element) is Emote:
                result = ChatEmote(
                    emote=element, src=element.url, height=self.font_size * 2
                )
            else:
                raise TypeError

            self.controls.append(result)

    async def subscribe_to_font_size_change(self, pubsub: PubSub):
        await pubsub.subscribe_all(
            [
                control.on_font_size_changed
                for control in self.controls
                if isinstance(control, FontSizeSubscriber)
            ]
        )

    async def subscribe_to_show_timestamp_change(self, pubsub: PubSub):
        await pubsub.subscribe_all(
            [
                control.on_show_timestamp_changed
                for control in self.controls
                if isinstance(control, ShowTimestampSubscriber)
            ]
        )



import logging
import ssl
from enum import StrEnum

import certifi
from aiohttp import ClientSession, TCPConnector

from hasherino.parse_irc import ParsedMessage


class HistorySource(StrEnum):
    ROBOTTY = "https://recent-messages.robotty.de/api/v2/recent-messages/{channel}"


async def get_chat_history(
    channel: str,
    limit: int = 100,
    source: HistorySource = HistorySource.ROBOTTY,
) -> list[ParsedMessage]:
    ssl_context = ssl.create_default_context(cafile=certifi.where())
    conn = TCPConnector(ssl=ssl_context)

    async with ClientSession(connector=conn) as session:
        async with session.get(
            source.value.format(channel=channel),
            params={"limit": limit},
        ) as response:
            json_result = await response.json()
            logging.debug(
                f"Get chat history for {channel} with limit {limit} returned response: {json_result}"
            )

            if not response.ok:
                message = f"Unable to get chat history for {channel} with response {json_result}"
                logging.debug(message)
                raise Exception(message)

            res = []

            for message in json_result["messages"]:
                # Add a : to messages loaded from history that have a single message,
                # since the message parser can only parse messages starting with a :
                cmd_idx = message.find("PRIVMSG")
                if -1 == cmd_idx:
                    continue
                msg_start_idx = message.find(" ", cmd_idx + len("PRIVMSG") + 2)
                if -1 == msg_start_idx:
                    continue
                msg_start_idx += 1
                if message[msg_start_idx] != ":":
                    message = message[:msg_start_idx] + ":" + message[msg_start_idx:]
                pm = ParsedMessage(message)
                res.append(pm)

            return res


async def main():
    for pm in await get_chat_history("hash_table"):
        print(pm.get_message_text())


if __name__ == "__main__":
    import asyncio

    asyncio.run(main())



import ssl

import certifi
from aiohttp import ClientSession, TCPConnector


class SevenTV:
    _EMOTES = {}

    @staticmethod
    def _get_client_session() -> ClientSession:
        ssl_context = ssl.create_default_context(cafile=certifi.where())
        conn = TCPConnector(ssl=ssl_context)
        return ClientSession(connector=conn)

    @staticmethod
    async def _gql_request(query: dict):
        async with SevenTV._get_client_session() as session:
            async with session.post(
                "https://7tv.io/v3/gql",
                headers={
                    "Content-Type": "application/json",
                },
                json=query,
            ) as response:
                return (await response.json())["data"]

    @staticmethod
    async def get_user(ttv_user_id: str):
        return (
            await SevenTV._gql_request(
                {
                    "operationName": "GetUserByConnection",
                    "query": """
                        query GetUserByConnection($platform: ConnectionPlatform! $id: String!) {
                            userByConnection (platform: $platform id: $id) {
                                id
                                type
                                username
                                roles
                                created_at
                                connections {
                                    id
                                    platform
                                    emote_set_id
                                }
                                editors {
                                  user {
                                    id
                                    username
                                  }
                                }
                                emote_sets {
                                    id
                                    emotes {
                                        id
                                        name
                                        data {
                                          id
                                          name
                                        }
                                    }
                                    capacity
                                }
                            }
                        }""",
                    "variables": {
                        "platform": "TWITCH",
                        "id": ttv_user_id,
                    },
                }
            )
        )["userByConnection"]

    @staticmethod
    async def get_global_emote_set() -> dict:
        if SevenTV._EMOTES:
            return SevenTV._EMOTES

        return (
            await SevenTV._gql_request(
                {
                    "operationName": "AppState",
                    "query": """
                        query AppState {
                          globalEmoteSet: namedEmoteSet(name: GLOBAL) {
                            id
                            name
                            emotes {
                              id
                              name
                              flags
                              __typename
                            }
                            capacity
                            __typename
                          }
                          roles: roles {
                            id
                            name
                            allowed
                            denied
                            position
                            color
                            invisible
                            __typename
                          }
                        }""",
                    "variables": {},
                }
            )
        )["globalEmoteSet"]
